__author__ = 'bvelez'
from Bio.Blast import NCBIWWW
from Bio.Blast import NCBIXML
from Bio import SeqIO
from sys import *
import csv

# Global constants
Directory=""
QueryName=['PAM70', 'BLOSUM62','BLOSUM80']
MatrixName=['PAM70', 'BLOSUM62','BLOSUM80']

def searchAndStoreBlastSearch(query, matrix, filename):
    'Performs a Blast search with the given query and substitution matrix and stores an XML result on a file named filename'
    # Creates handle to store the results sent by NCBI website
    results_handle = NCBIWWW.qblast("blastp", "swissprot", query, expect=10,
                                    descriptions=2000, alignments=2000, hitlist_size=2000,
                                    matrix_name=matrix)
    # Reads results into memory
    blast_results = results_handle.read()
    # Creates and opens a file in filesystem for writing
    save_file = open(filename, 'w')
    # Store results from memory into the filesystem
    save_file.write(blast_results)
    #Close the file handler
    save_file.close()

def xmlToTSV(xmlFilename, tsvFileName, matrix):
    'Parses the input XML file and generates a TSV file with a row for each high scoring pairs'
    # Create file handler
    blast_fileptr=open(xmlFilename)
    # Parse xml file into biopython object
    record=NCBIXML.parse(blast_fileptr).next()
    # Close file handler
    blast_fileptr.close()
    # Create tsv file writer
    tsvFileWriter = csv.writer(open(tsvFileName,'a'), delimiter='\t')
    # Iterate over record alignments
    for alignment in record.alignments:
        # Extract high score pairwise alignment object
        hspIndex = 0
        for hsp in alignment.hsps:
            hsp=alignment.hsps[0]
            # Create record vector
            record = [alignment.accession, alignment.title, matrix, hspIndex, hsp.expect, hsp.score]
            # Store record vector into tsv file
            tsvFileWriter.writerow( record )

def doIt(fastaFileName):
    if (fastaFileName[-6:] != ".fasta"):
        print ("Input file must have .fasta extension")
        exit(1)
    fastaFile = open(fastaFileName)
    print ("Parsing input query file in Fasta format: " + fastaFileName)
    #fasta = SeqIO.parse(fastaFile,"fasta")
    seqObject = SeqIO.read(fastaFile, format="fasta").seq
    print ("Blastp on query string [" + seqObject + "]")
    fastaFile.close()
    tsvFileName=fastaFileName.strip(".fasta") + ".tsv"
    temp=open(tsvFileName,"w") # Erase TSV file
    temp.close
    for matrix in MatrixName:
        xmlFileName=fastaFileName.strip(".fasta") + "_" + matrix + ".xml"
        print ("Running BLAST search ... matrix=" + matrix)
        searchAndStoreBlastSearch(seqObject, matrix, xmlFileName)
        print ("Appending results to output TSV file ... matrix=" + matrix)
        xmlToTSV(xmlFileName, tsvFileName, matrix)
        print ("Output TSV file stored in: " + tsvFileName)


doIt("/Users/bvelez/Sites/ece/courses/MARC/RelationalDatabases2015/SourceCode/P00624.fasta")